﻿// This work contains valuable confidential and proprietary information. 
// Disclosure, use, or reproduction without the written authorization of 
// Vitaliy Leschenko is strictly prohibited. This unpublished work 
// is Copyright (c) and protected by the laws. 
// 
// If publication of the work should occur, the following notice shall apply: 
// Copyright (c) 2009 Vitaliy Leschenko. All rights reserved.
//
// $Id: MainPage.xaml.cs 518 2009-09-07 03:34:33Z v.leschenko $ 
//

using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Threading;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Linq;
using Axelerate.Silverlight3D.Media.Media3D;
using System.Text;

namespace Mercury.Web.Silverlight.Tags
{
    public partial class MainPage
    {
        private readonly RotateTransform3D rotateTransform = new RotateTransform3D();
        private bool runRotation;
        private double slowDownCounter = 500.0;
        private List<Tag3D> tagBlocks;
        private readonly List<string> tagList = new List<string>();

        public MainPage()
        {
            InitializeComponent();
            Loaded += OnTagCloudLoaded;
            SizeChanged += OnPageSizeChanged;
        }

        private void OnPageSizeChanged(object sender, SizeChangedEventArgs e)
        {
            if (tagBlocks != null)
            {
                var size = GetUISizes();

                foreach (var tagd in tagBlocks)
                {
                    tagd.Redraw(size);
                }
            }
        }

        private void OnCompositionTargetRendering(object sender, EventArgs e)
        {
            if (!(runRotation || (slowDownCounter <= 0.0)))
            {
                var rotation = (AxisAngleRotation3D)rotateTransform.Rotation;
                rotation.Angle *= slowDownCounter / 500.0;
                rotateTransform.Rotation = rotation;
                slowDownCounter--;
            }
            if (((AxisAngleRotation3D)rotateTransform.Rotation).Angle > 0.05)
            {
                RotateBlocks();
            }
        }

        private void FillTags()
        {
            RootCanvas.UpdateLayout();
            RootCanvas.Children.Clear();
            tagBlocks = new List<Tag3D>();

            var length = tagList.Count;
            for (var i = 1; i <= length; i++)
            {
                var a = Math.Acos(-1.0 + (2.0 * i - 1.0) / length);
                var d = Math.Sqrt(length * Math.PI) * a;
                var x = Math.Cos(d) * Math.Sin(a);
                var y = Math.Sin(d) * Math.Sin(a);
                var z = Math.Cos(a);

                var tag = tagList[i - 1];
                var uri = UrlHelper.GetTagUri(UrlEncode(tag));

                var item = new Tag3D(x, y, z, tag, uri);
                RootCanvas.Children.Add(item.BtnLink);
                tagBlocks.Add(item);
            }
        }

        private static string UrlEncode(string tag)
        {
            var bytes = Encoding.UTF8.GetBytes(tag);

            var result = new StringBuilder();
            foreach(var ch in bytes)
            {
                result.AppendFormat("%{0:x2}", ch);
            }
            return result.ToString();
        }

        private void OnLayoutRootMouseEnter(object sender, MouseEventArgs e)
        {
            LayoutRoot.MouseMove += OnLayoutRootMouseMove;
            runRotation = true;
            slowDownCounter = 500.0;
        }

        private void OnLayoutRootMouseLeave(object sender, MouseEventArgs e)
        {
            LayoutRoot.MouseMove -= OnLayoutRootMouseMove;
            runRotation = false;
        }

        private void OnLayoutRootMouseMove(object sender, MouseEventArgs e)
        {
            var position = e.GetPosition(RootCanvas);
            SetRotateTransform(position);
        }

        private void RotateBlocks()
        {
            var size = GetUISizes();

            foreach (var tagd in tagBlocks)
            {
                Point3D pointd;
                if (rotateTransform.TryTransform(tagd.CenterPoint, out pointd))
                {
                    tagd.CenterPoint = pointd;
                    tagd.Redraw(size);
                }
            }
        }

        private void OnTagCloudLoaded(object sender, RoutedEventArgs e)
        {
            RootCanvas.UpdateLayout();

            var uri = UrlHelper.GetUri(((App)Application.Current).ServiceUrl);
            new Thread(obj => LoadTagList((Uri)obj)).Start(uri);
        }

        private void LoadTagList(Uri uri)
        {
            try
            {
                var request = (HttpWebRequest)WebRequest.Create(uri);

                var waitingEvent = new AutoResetEvent(false);
                var callback = new AsyncCallback(result => ((EventWaitHandle)result.AsyncState).Set());

                var asyncResult = request.BeginGetResponse(callback, waitingEvent);
                waitingEvent.WaitOne();

                var response = request.EndGetResponse(asyncResult);

                var doc = XDocument.Load(response.GetResponseStream());
                tagList.AddRange(doc.Descendants("tag").Select(item => item.Value));

                Dispatcher.BeginInvoke(Run);
            }
            catch
            {
            }
        }

        public void Run()
        {
            CompositionTarget.Rendering += OnCompositionTargetRendering;
            LayoutRoot.MouseEnter += OnLayoutRootMouseEnter;
            LayoutRoot.MouseLeave += OnLayoutRootMouseLeave;

            slowDownCounter = 500.0;
            runRotation = true;
            rotateTransform.Rotation = new AxisAngleRotation3D(new Vector3D(0.8, 0.6, 0), 0.5);

            FillTags();
        }

        private void SetRotateTransform(Point position)
        {
            var size = GetUISizes();

            var x = (position.X - size.XOffset) / size.XRadius;
            var y = (position.Y - size.YOffset) / size.YRadius;
            var angle = Math.Sqrt(x * x + y * y);
            rotateTransform.Rotation = new AxisAngleRotation3D(new Vector3D(-y, -x, 0.0), angle);
        }

        private UISize GetUISizes()
        {
            return new UISize
                {
                    XOffset = RootCanvas.ActualWidth / 2.0,
                    YOffset = RootCanvas.ActualHeight / 2.0
                };
        }
    }
}
