﻿// This work contains valuable confidential and proprietary information. 
// Disclosure, use, or reproduction without the written authorization of 
// Vitaliy Leschenko is strictly prohibited. This unpublished work 
// is Copyright (c) and protected by the laws. 
// 
// If publication of the work should occur, the following notice shall apply: 
// Copyright (c) 2009 Vitaliy Leschenko. All rights reserved.
//
// $Id: Tag3D.cs 518 2009-09-07 03:34:33Z v.leschenko $ 
//

using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using Axelerate.Silverlight3D.Media.Media3D;

namespace Mercury.Web.Silverlight.Tags
{
    public class Tag3D
    {
        public Tag3D(double x, double y, double z, string text, Uri uri)
        {
            CenterPoint = new Point3D(x, y, z);
            TextBlock = new TextBlock { Text = text };
            BtnLink = new HyperlinkButton
                      {
                          Content = TextBlock,
                          BorderThickness = new Thickness(0),
                          Padding = new Thickness(0),
                          NavigateUri = uri,
                          Visibility = Visibility.Collapsed
                      };
        }

        public HyperlinkButton BtnLink { get; set; }
        public Point3D CenterPoint { get; set; }
        private TextBlock TextBlock { get; set; }

        public void Redraw(UISize size)
        {
            UpdateLayout();

            UpdateSize();
            UpdateColor();
            UpdateVisibility();
            UpdatePosition(size.XRadius, size.YRadius, size.XOffset, size.YOffset);
        }

        private void UpdatePosition(double rx, double ry, double xOffset, double yOffset) 
        {
            var maxZ = Math.Min(rx, ry);
            var x = (xOffset + CenterPoint.X * rx) - (BtnLink.ActualWidth / 2.0);
            var y = (yOffset - CenterPoint.Y * ry) - (BtnLink.ActualHeight / 2.0);
            var z = CenterPoint.Z * maxZ;

            Canvas.SetLeft(BtnLink, x);
            Canvas.SetTop(BtnLink, y);
            Canvas.SetZIndex(BtnLink, Convert.ToInt32(z));
        }

        private void UpdateLayout()
        {
            BtnLink.UpdateLayout();
        }

        private void UpdateSize()
        {
            BtnLink.FontSize = 16 + CenterPoint.Z * 4;
        }

        private void UpdateColor() 
        {
            var color = 160 + CenterPoint.Z * 96;
            color = Math.Max(0, color);
            color = Math.Min(255, color);
            BtnLink.Foreground = new SolidColorBrush(Color.FromArgb(Convert.ToByte(color), 0, 0, 0));
        }

        private void UpdateVisibility()
        {
            if (BtnLink.Visibility == Visibility.Collapsed)
            {
                BtnLink.Visibility = Visibility.Visible;
            }
        }
    }
}